#ifndef GEN_UI_GFX_X_XPROTO_H_
#define GEN_UI_GFX_X_XPROTO_H_

#include <array>
#include <cstddef>
#include <cstdint>
#include <cstring>
#include <vector>

#include "base/component_export.h"
#include "ui/gfx/x/xproto_types.h"
#include "xproto_undef.h"

typedef struct _XDisplay XDisplay;

namespace x11 {

class COMPONENT_EXPORT(X11) XProto {
  public:
  explicit XProto(XDisplay* display);

  XDisplay* display() { return display_; }

  enum class GraphicsContext : uint32_t {};

  enum class ColorMap : uint32_t {};

  enum class Drawable : uint32_t {};

  enum class Fontable : uint32_t {};

  enum class Bool32 : uint32_t {};

  enum class VisualId : uint32_t {};

  enum class TimeStamp : uint32_t {};

  enum class KeySym : uint32_t {};

  enum class KeyCode : uint8_t {};

  enum class KeyCode32 : uint32_t {};

  enum class Button : uint8_t {};

  enum class VisualClass : int {
    StaticGray = 0,
    GrayScale = 1,
    StaticColor = 2,
    PseudoColor = 3,
    TrueColor = 4,
    DirectColor = 5,
  };

  enum class EventMask : int {
    NoEvent = 0,
    KeyPress = 1 << 0,
    KeyRelease = 1 << 1,
    ButtonPress = 1 << 2,
    ButtonRelease = 1 << 3,
    EnterWindow = 1 << 4,
    LeaveWindow = 1 << 5,
    PointerMotion = 1 << 6,
    PointerMotionHint = 1 << 7,
    Button1Motion = 1 << 8,
    Button2Motion = 1 << 9,
    Button3Motion = 1 << 10,
    Button4Motion = 1 << 11,
    Button5Motion = 1 << 12,
    ButtonMotion = 1 << 13,
    KeymapState = 1 << 14,
    Exposure = 1 << 15,
    VisibilityChange = 1 << 16,
    StructureNotify = 1 << 17,
    ResizeRedirect = 1 << 18,
    SubstructureNotify = 1 << 19,
    SubstructureRedirect = 1 << 20,
    FocusChange = 1 << 21,
    PropertyChange = 1 << 22,
    ColorMapChange = 1 << 23,
    OwnerGrabButton = 1 << 24,
  };

  enum class BackingStore : int {
    NotUseful = 0,
    WhenMapped = 1,
    Always = 2,
  };

  enum class ImageOrder : int {
    LSBFirst = 0,
    MSBFirst = 1,
  };

  enum class ModMask : int {
    Shift = 1 << 0,
    Lock = 1 << 1,
    Control = 1 << 2,
    c_1 = 1 << 3,
    c_2 = 1 << 4,
    c_3 = 1 << 5,
    c_4 = 1 << 6,
    c_5 = 1 << 7,
    Any = 1 << 15,
  };

  enum class KeyButMask : int {
    Shift = 1 << 0,
    Lock = 1 << 1,
    Control = 1 << 2,
    Mod1 = 1 << 3,
    Mod2 = 1 << 4,
    Mod3 = 1 << 5,
    Mod4 = 1 << 6,
    Mod5 = 1 << 7,
    Button1 = 1 << 8,
    Button2 = 1 << 9,
    Button3 = 1 << 10,
    Button4 = 1 << 11,
    Button5 = 1 << 12,
  };

  enum class Window : uint32_t {
    None = 0,
  };

  enum class ButtonMask : int {
    c_1 = 1 << 8,
    c_2 = 1 << 9,
    c_3 = 1 << 10,
    c_4 = 1 << 11,
    c_5 = 1 << 12,
    Any = 1 << 15,
  };

  enum class Motion : int {
    Normal = 0,
    Hint = 1,
  };

  enum class NotifyDetail : int {
    Ancestor = 0,
    Virtual = 1,
    Inferior = 2,
    Nonlinear = 3,
    NonlinearVirtual = 4,
    Pointer = 5,
    PointerRoot = 6,
    None = 7,
  };

  enum class NotifyMode : int {
    Normal = 0,
    Grab = 1,
    Ungrab = 2,
    WhileGrabbed = 3,
  };

  enum class Visibility : int {
    Unobscured = 0,
    PartiallyObscured = 1,
    FullyObscured = 2,
  };

  enum class Place : int {
    OnTop = 0,
    OnBottom = 1,
  };

  enum class Property : int {
    NewValue = 0,
    Delete = 1,
  };

  enum class Time : int {
    CurrentTime = 0,
  };

  enum class Atom : uint32_t {
    None = 0,
    Any = 0,
    PRIMARY = 1,
    SECONDARY = 2,
    ARC = 3,
    ATOM = 4,
    BITMAP = 5,
    CARDINAL = 6,
    COLORMAP = 7,
    CURSOR = 8,
    CUT_BUFFER0 = 9,
    CUT_BUFFER1 = 10,
    CUT_BUFFER2 = 11,
    CUT_BUFFER3 = 12,
    CUT_BUFFER4 = 13,
    CUT_BUFFER5 = 14,
    CUT_BUFFER6 = 15,
    CUT_BUFFER7 = 16,
    DRAWABLE = 17,
    FONT = 18,
    INTEGER = 19,
    PIXMAP = 20,
    POINT = 21,
    RECTANGLE = 22,
    RESOURCE_MANAGER = 23,
    RGB_COLOR_MAP = 24,
    RGB_BEST_MAP = 25,
    RGB_BLUE_MAP = 26,
    RGB_DEFAULT_MAP = 27,
    RGB_GRAY_MAP = 28,
    RGB_GREEN_MAP = 29,
    RGB_RED_MAP = 30,
    STRING = 31,
    VISUALID = 32,
    WINDOW = 33,
    WM_COMMAND = 34,
    WM_HINTS = 35,
    WM_CLIENT_MACHINE = 36,
    WM_ICON_NAME = 37,
    WM_ICON_SIZE = 38,
    WM_NAME = 39,
    WM_NORMAL_HINTS = 40,
    WM_SIZE_HINTS = 41,
    WM_ZOOM_HINTS = 42,
    MIN_SPACE = 43,
    NORM_SPACE = 44,
    MAX_SPACE = 45,
    END_SPACE = 46,
    SUPERSCRIPT_X = 47,
    SUPERSCRIPT_Y = 48,
    SUBSCRIPT_X = 49,
    SUBSCRIPT_Y = 50,
    UNDERLINE_POSITION = 51,
    UNDERLINE_THICKNESS = 52,
    STRIKEOUT_ASCENT = 53,
    STRIKEOUT_DESCENT = 54,
    ITALIC_ANGLE = 55,
    X_HEIGHT = 56,
    QUAD_WIDTH = 57,
    WEIGHT = 58,
    POINT_SIZE = 59,
    RESOLUTION = 60,
    COPYRIGHT = 61,
    NOTICE = 62,
    FONT_NAME = 63,
    FAMILY_NAME = 64,
    FULL_NAME = 65,
    CAP_HEIGHT = 66,
    WM_CLASS = 67,
    WM_TRANSIENT_FOR = 68,
  };

  enum class ColormapState : int {
    Uninstalled = 0,
    Installed = 1,
  };

  enum class Colormap : int {
    None = 0,
  };

  enum class Mapping : int {
    Modifier = 0,
    Keyboard = 1,
    Pointer = 2,
  };

  enum class WindowClass : int {
    CopyFromParent = 0,
    InputOutput = 1,
    InputOnly = 2,
  };

  enum class CreateWindowAttribute : int {
    BackPixmap = 1 << 0,
    BackPixel = 1 << 1,
    BorderPixmap = 1 << 2,
    BorderPixel = 1 << 3,
    BitGravity = 1 << 4,
    WinGravity = 1 << 5,
    BackingStore = 1 << 6,
    BackingPlanes = 1 << 7,
    BackingPixel = 1 << 8,
    OverrideRedirect = 1 << 9,
    SaveUnder = 1 << 10,
    EventMask = 1 << 11,
    DontPropagate = 1 << 12,
    Colormap = 1 << 13,
    Cursor = 1 << 14,
  };

  enum class BackPixmap : int {
    None = 0,
    ParentRelative = 1,
  };

  enum class Gravity : int {
    BitForget = 0,
    WinUnmap = 0,
    NorthWest = 1,
    North = 2,
    NorthEast = 3,
    West = 4,
    Center = 5,
    East = 6,
    SouthWest = 7,
    South = 8,
    SouthEast = 9,
    Static = 10,
  };

  enum class MapState : int {
    Unmapped = 0,
    Unviewable = 1,
    Viewable = 2,
  };

  enum class SetMode : int {
    Insert = 0,
    Delete = 1,
  };

  enum class ConfigWindow : int {
    X = 1 << 0,
    Y = 1 << 1,
    Width = 1 << 2,
    Height = 1 << 3,
    BorderWidth = 1 << 4,
    Sibling = 1 << 5,
    StackMode = 1 << 6,
  };

  enum class StackMode : int {
    Above = 0,
    Below = 1,
    TopIf = 2,
    BottomIf = 3,
    Opposite = 4,
  };

  enum class Circulate : int {
    RaiseLowest = 0,
    LowerHighest = 1,
  };

  enum class PropMode : int {
    Replace = 0,
    Prepend = 1,
    Append = 2,
  };

  enum class GetPropertyType : int {
    Any = 0,
  };

  enum class SendEventDest : int {
    PointerWindow = 0,
    ItemFocus = 1,
  };

  enum class GrabMode : int {
    Sync = 0,
    Async = 1,
  };

  enum class GrabStatus : int {
    Success = 0,
    AlreadyGrabbed = 1,
    InvalidTime = 2,
    NotViewable = 3,
    Frozen = 4,
  };

  enum class Cursor : uint32_t {
    None = 0,
  };

  enum class ButtonIndex : int {
    Any = 0,
    c_1 = 1,
    c_2 = 2,
    c_3 = 3,
    c_4 = 4,
    c_5 = 5,
  };

  enum class Grab : int {
    Any = 0,
  };

  enum class Allow : int {
    AsyncPointer = 0,
    SyncPointer = 1,
    ReplayPointer = 2,
    AsyncKeyboard = 3,
    SyncKeyboard = 4,
    ReplayKeyboard = 5,
    AsyncBoth = 6,
    SyncBoth = 7,
  };

  enum class InputFocus : int {
    None = 0,
    PointerRoot = 1,
    Parent = 2,
    FollowKeyboard = 3,
  };

  enum class FontDraw : int {
    LeftToRight = 0,
    RightToLeft = 1,
  };

  enum class GraphicsContextAttribute : int {
    Function = 1 << 0,
    PlaneMask = 1 << 1,
    Foreground = 1 << 2,
    Background = 1 << 3,
    LineWidth = 1 << 4,
    LineStyle = 1 << 5,
    CapStyle = 1 << 6,
    JoinStyle = 1 << 7,
    FillStyle = 1 << 8,
    FillRule = 1 << 9,
    Tile = 1 << 10,
    Stipple = 1 << 11,
    TileStippleOriginX = 1 << 12,
    TileStippleOriginY = 1 << 13,
    Font = 1 << 14,
    SubwindowMode = 1 << 15,
    GraphicsExposures = 1 << 16,
    ClipOriginX = 1 << 17,
    ClipOriginY = 1 << 18,
    ClipMask = 1 << 19,
    DashOffset = 1 << 20,
    DashList = 1 << 21,
    ArcMode = 1 << 22,
  };

  enum class Gx : int {
    clear = 0,
    c_and = 1,
    andReverse = 2,
    copy = 3,
    andInverted = 4,
    noop = 5,
    c_xor = 6,
    c_or = 7,
    nor = 8,
    equiv = 9,
    invert = 10,
    orReverse = 11,
    copyInverted = 12,
    orInverted = 13,
    nand = 14,
    set = 15,
  };

  enum class LineStyle : int {
    Solid = 0,
    OnOffDash = 1,
    DoubleDash = 2,
  };

  enum class CapStyle : int {
    NotLast = 0,
    Butt = 1,
    Round = 2,
    Projecting = 3,
  };

  enum class JoinStyle : int {
    Miter = 0,
    Round = 1,
    Bevel = 2,
  };

  enum class FillStyle : int {
    Solid = 0,
    Tiled = 1,
    Stippled = 2,
    OpaqueStippled = 3,
  };

  enum class FillRule : int {
    EvenOdd = 0,
    Winding = 1,
  };

  enum class SubwindowMode : int {
    ClipByChildren = 0,
    IncludeInferiors = 1,
  };

  enum class ArcMode : int {
    Chord = 0,
    PieSlice = 1,
  };

  enum class ClipOrdering : int {
    Unsorted = 0,
    YSorted = 1,
    YXSorted = 2,
    YXBanded = 3,
  };

  enum class CoordMode : int {
    Origin = 0,
    Previous = 1,
  };

  enum class PolyShape : int {
    Complex = 0,
    Nonconvex = 1,
    Convex = 2,
  };

  enum class ImageFormat : int {
    XYBitmap = 0,
    XYPixmap = 1,
    ZPixmap = 2,
  };

  enum class ColormapAlloc : int {
    None = 0,
    All = 1,
  };

  enum class ColorFlag : int {
    Red = 1 << 0,
    Green = 1 << 1,
    Blue = 1 << 2,
  };

  enum class Pixmap : uint32_t {
    None = 0,
  };

  enum class Font : uint32_t {
    None = 0,
  };

  enum class QueryShapeOf : int {
    LargestCursor = 0,
    FastestTile = 1,
    FastestStipple = 2,
  };

  enum class Keyboard : int {
    KeyClickPercent = 1 << 0,
    BellPercent = 1 << 1,
    BellPitch = 1 << 2,
    BellDuration = 1 << 3,
    Led = 1 << 4,
    LedMode = 1 << 5,
    Key = 1 << 6,
    AutoRepeatMode = 1 << 7,
  };

  enum class LedMode : int {
    Off = 0,
    On = 1,
  };

  enum class AutoRepeatMode : int {
    Off = 0,
    On = 1,
    Default = 2,
  };

  enum class Blanking : int {
    NotPreferred = 0,
    Preferred = 1,
    Default = 2,
  };

  enum class Exposures : int {
    NotAllowed = 0,
    Allowed = 1,
    Default = 2,
  };

  enum class HostMode : int {
    Insert = 0,
    Delete = 1,
  };

  enum class Family : int {
    Internet = 0,
    DECnet = 1,
    Chaos = 2,
    ServerInterpreted = 5,
    Internet6 = 6,
  };

  enum class AccessControl : int {
    Disable = 0,
    Enable = 1,
  };

  enum class CloseDown : int {
    DestroyAll = 0,
    RetainPermanent = 1,
    RetainTemporary = 2,
  };

  enum class Kill : int {
    AllTemporary = 0,
  };

  enum class ScreenSaver : int {
    Reset = 0,
    Active = 1,
  };

  enum class MappingStatus : int {
    Success = 0,
    Busy = 1,
    Failure = 2,
  };

  enum class MapIndex : int {
    Shift = 0,
    Lock = 1,
    Control = 2,
    c_1 = 3,
    c_2 = 4,
    c_3 = 5,
    c_4 = 6,
    c_5 = 7,
  };

  struct Char16 {
    uint8_t byte1{};
    uint8_t byte2{};
  };

  struct Point {
    int16_t x{};
    int16_t y{};
  };

  struct Rectangle {
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
  };

  struct Arc {
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    int16_t angle1{};
    int16_t angle2{};
  };

  struct Format {
    uint8_t depth{};
    uint8_t bits_per_pixel{};
    uint8_t scanline_pad{};
  };

  struct VisualType {
    VisualId visual_id{};
    VisualClass c_class{};
    uint8_t bits_per_rgb_value{};
    uint16_t colormap_entries{};
    uint32_t red_mask{};
    uint32_t green_mask{};
    uint32_t blue_mask{};
  };

  struct Depth {
    uint8_t depth{};
    std::vector<VisualType> visuals{};
  };

  struct Screen {
    Window root{};
    ColorMap default_colormap{};
    uint32_t white_pixel{};
    uint32_t black_pixel{};
    EventMask current_input_masks{};
    uint16_t width_in_pixels{};
    uint16_t height_in_pixels{};
    uint16_t width_in_millimeters{};
    uint16_t height_in_millimeters{};
    uint16_t min_installed_maps{};
    uint16_t max_installed_maps{};
    VisualId root_visual{};
    BackingStore backing_stores{};
    uint8_t save_unders{};
    uint8_t root_depth{};
    uint8_t allowed_depths_len{};
    std::vector<Depth> allowed_depths{};
  };

  struct SetupRequest {
    uint8_t byte_order{};
    uint16_t protocol_major_version{};
    uint16_t protocol_minor_version{};
    uint16_t authorization_protocol_name_len{};
    uint16_t authorization_protocol_data_len{};
    std::string authorization_protocol_name{};
    std::string authorization_protocol_data{};
  };

  struct SetupFailed {
    uint8_t status{};
    uint8_t reason_len{};
    uint16_t protocol_major_version{};
    uint16_t protocol_minor_version{};
    uint16_t length{};
    std::string reason{};
  };

  struct SetupAuthenticate {
    uint8_t status{};
    uint16_t length{};
    std::string reason{};
  };

  struct Setup {
    uint8_t status{};
    uint16_t protocol_major_version{};
    uint16_t protocol_minor_version{};
    uint16_t length{};
    uint32_t release_number{};
    uint32_t resource_id_base{};
    uint32_t resource_id_mask{};
    uint32_t motion_buffer_size{};
    uint16_t vendor_len{};
    uint16_t maximum_request_length{};
    uint8_t roots_len{};
    ImageOrder image_byte_order{};
    ImageOrder bitmap_format_bit_order{};
    uint8_t bitmap_format_scanline_unit{};
    uint8_t bitmap_format_scanline_pad{};
    KeyCode min_keycode{};
    KeyCode max_keycode{};
    std::string vendor{};
    std::vector<Format> pixmap_formats{};
    std::vector<Screen> roots{};
  };

  struct KeyPressEvent {
    KeyCode detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    uint8_t same_screen{};
  };

  struct KeyReleaseEvent {
    KeyCode detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    uint8_t same_screen{};
  };

  struct ButtonPressEvent {
    Button detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    uint8_t same_screen{};
  };

  struct ButtonReleaseEvent {
    Button detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    uint8_t same_screen{};
  };

  struct MotionNotifyEvent {
    Motion detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    uint8_t same_screen{};
  };

  struct EnterNotifyEvent {
    NotifyDetail detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    NotifyMode mode{};
    uint8_t same_screen_focus{};
  };

  struct LeaveNotifyEvent {
    NotifyDetail detail{};
    TimeStamp time{};
    Window root{};
    Window event{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t event_x{};
    int16_t event_y{};
    KeyButMask state{};
    NotifyMode mode{};
    uint8_t same_screen_focus{};
  };

  struct FocusInEvent {
    NotifyDetail detail{};
    Window event{};
    NotifyMode mode{};
  };

  struct FocusOutEvent {
    NotifyDetail detail{};
    Window event{};
    NotifyMode mode{};
  };

  struct KeymapNotifyEvent {
    std::array<uint8_t, 31> keys{};
  };

  struct ExposeEvent {
    Window window{};
    uint16_t x{};
    uint16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t count{};
  };

  struct GraphicsExposureEvent {
    Drawable drawable{};
    uint16_t x{};
    uint16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t minor_opcode{};
    uint16_t count{};
    uint8_t major_opcode{};
  };

  struct NoExposureEvent {
    Drawable drawable{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct VisibilityNotifyEvent {
    Window window{};
    Visibility state{};
  };

  struct CreateNotifyEvent {
    Window parent{};
    Window window{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t border_width{};
    uint8_t override_redirect{};
  };

  struct DestroyNotifyEvent {
    Window event{};
    Window window{};
  };

  struct UnmapNotifyEvent {
    Window event{};
    Window window{};
    uint8_t from_configure{};
  };

  struct MapNotifyEvent {
    Window event{};
    Window window{};
    uint8_t override_redirect{};
  };

  struct MapRequestEvent {
    Window parent{};
    Window window{};
  };

  struct ReparentNotifyEvent {
    Window event{};
    Window window{};
    Window parent{};
    int16_t x{};
    int16_t y{};
    uint8_t override_redirect{};
  };

  struct ConfigureNotifyEvent {
    Window event{};
    Window window{};
    Window above_sibling{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t border_width{};
    uint8_t override_redirect{};
  };

  struct ConfigureRequestEvent {
    StackMode stack_mode{};
    Window parent{};
    Window window{};
    Window sibling{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t border_width{};
    ConfigWindow value_mask{};
  };

  struct GravityNotifyEvent {
    Window event{};
    Window window{};
    int16_t x{};
    int16_t y{};
  };

  struct ResizeRequestEvent {
    Window window{};
    uint16_t width{};
    uint16_t height{};
  };

  struct CirculateNotifyEvent {
    Window event{};
    Window window{};
    Place place{};
  };

  struct CirculateRequestEvent {
    Window event{};
    Window window{};
    Place place{};
  };

  struct PropertyNotifyEvent {
    Window window{};
    Atom atom{};
    TimeStamp time{};
    Property state{};
  };

  struct SelectionClearEvent {
    TimeStamp time{};
    Window owner{};
    Atom selection{};
  };

  struct SelectionRequestEvent {
    TimeStamp time{};
    Window owner{};
    Window requestor{};
    Atom selection{};
    Atom target{};
    Atom property{};
  };

  struct SelectionNotifyEvent {
    TimeStamp time{};
    Window requestor{};
    Atom selection{};
    Atom target{};
    Atom property{};
  };

  struct ColormapNotifyEvent {
    Window window{};
    ColorMap colormap{};
    uint8_t c_new{};
    ColormapState state{};
  };

  union ClientMessageData {
    ClientMessageData() { memset(this, 0, sizeof(*this)); }

    uint8_t data8;
    uint16_t data16;
    uint32_t data32;
  };
  static_assert(std::is_trivially_copyable<ClientMessageData>::value, "");

  struct ClientMessageEvent {
    uint8_t format{};
    Window window{};
    Atom type{};
    ClientMessageData data{};
  };

  struct MappingNotifyEvent {
    Mapping request{};
    KeyCode first_keycode{};
    uint8_t count{};
  };

  struct GeGenericEvent {
  };

  struct RequestError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct ValueError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct WindowError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct PixmapError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct AtomError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct CursorError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct FontError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct MatchError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct DrawableError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct AccessError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct AllocError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct ColormapError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct GContextError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct IDChoiceError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct NameError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct LengthError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct ImplementationError {
    uint32_t bad_value{};
    uint16_t minor_opcode{};
    uint8_t major_opcode{};
  };

  struct TimeCoord {
    TimeStamp time{};
    int16_t x{};
    int16_t y{};
  };

  struct FontProperty {
    Atom name{};
    uint32_t value{};
  };

  struct CharInfo {
    int16_t left_side_bearing{};
    int16_t right_side_bearing{};
    int16_t character_width{};
    int16_t ascent{};
    int16_t descent{};
    uint16_t attributes{};
  };

  struct Str {
    std::string name{};
  };

  struct SEGMENT {
    int16_t x1{};
    int16_t y1{};
    int16_t x2{};
    int16_t y2{};
  };

  struct ColorItem {
    uint32_t pixel{};
    uint16_t red{};
    uint16_t green{};
    uint16_t blue{};
    ColorFlag flags{};
  };

  struct Rgb {
    uint16_t red{};
    uint16_t green{};
    uint16_t blue{};
  };

  struct Host {
    Family family{};
    uint16_t address_len{};
    std::vector<uint8_t> address{};
  };

  struct CreateWindowRequest {
    uint8_t depth{};
    Window wid{};
    Window parent{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t border_width{};
    WindowClass c_class{};
    VisualId visual{};
    CreateWindowAttribute value_mask{};
    struct {
      Pixmap background_pixmap{};
      uint32_t background_pixel{};
      Pixmap border_pixmap{};
      uint32_t border_pixel{};
      Gravity bit_gravity{};
      Gravity win_gravity{};
      BackingStore backing_store{};
      uint32_t backing_planes{};
      uint32_t backing_pixel{};
      Bool32 override_redirect{};
      Bool32 save_under{};
      EventMask event_mask{};
      EventMask do_not_propogate_mask{};
      ColorMap colormap{};
      Cursor cursor{};
    } value_list;
  };

  using CreateWindowResponse = Response<void>;

  Future<void> CreateWindow(
      const CreateWindowRequest& request);

  struct ChangeWindowAttributesRequest {
    Window window{};
    CreateWindowAttribute value_mask{};
    struct {
      Pixmap background_pixmap{};
      uint32_t background_pixel{};
      Pixmap border_pixmap{};
      uint32_t border_pixel{};
      Gravity bit_gravity{};
      Gravity win_gravity{};
      BackingStore backing_store{};
      uint32_t backing_planes{};
      uint32_t backing_pixel{};
      Bool32 override_redirect{};
      Bool32 save_under{};
      EventMask event_mask{};
      EventMask do_not_propogate_mask{};
      ColorMap colormap{};
      Cursor cursor{};
    } value_list;
  };

  using ChangeWindowAttributesResponse = Response<void>;

  Future<void> ChangeWindowAttributes(
      const ChangeWindowAttributesRequest& request);

  struct GetWindowAttributesRequest {
    Window window{};
  };

  struct GetWindowAttributesReply {
    BackingStore backing_store{};
    VisualId visual{};
    WindowClass c_class{};
    Gravity bit_gravity{};
    Gravity win_gravity{};
    uint32_t backing_planes{};
    uint32_t backing_pixel{};
    uint8_t save_under{};
    uint8_t map_is_installed{};
    MapState map_state{};
    uint8_t override_redirect{};
    ColorMap colormap{};
    EventMask all_event_masks{};
    EventMask your_event_mask{};
    EventMask do_not_propagate_mask{};
  };

  using GetWindowAttributesResponse = Response<GetWindowAttributesReply>;

  Future<GetWindowAttributesReply> GetWindowAttributes(
      const GetWindowAttributesRequest& request);

  struct DestroyWindowRequest {
    Window window{};
  };

  using DestroyWindowResponse = Response<void>;

  Future<void> DestroyWindow(
      const DestroyWindowRequest& request);

  struct DestroySubwindowsRequest {
    Window window{};
  };

  using DestroySubwindowsResponse = Response<void>;

  Future<void> DestroySubwindows(
      const DestroySubwindowsRequest& request);

  struct ChangeSaveSetRequest {
    SetMode mode{};
    Window window{};
  };

  using ChangeSaveSetResponse = Response<void>;

  Future<void> ChangeSaveSet(
      const ChangeSaveSetRequest& request);

  struct ReparentWindowRequest {
    Window window{};
    Window parent{};
    int16_t x{};
    int16_t y{};
  };

  using ReparentWindowResponse = Response<void>;

  Future<void> ReparentWindow(
      const ReparentWindowRequest& request);

  struct MapWindowRequest {
    Window window{};
  };

  using MapWindowResponse = Response<void>;

  Future<void> MapWindow(
      const MapWindowRequest& request);

  struct MapSubwindowsRequest {
    Window window{};
  };

  using MapSubwindowsResponse = Response<void>;

  Future<void> MapSubwindows(
      const MapSubwindowsRequest& request);

  struct UnmapWindowRequest {
    Window window{};
  };

  using UnmapWindowResponse = Response<void>;

  Future<void> UnmapWindow(
      const UnmapWindowRequest& request);

  struct UnmapSubwindowsRequest {
    Window window{};
  };

  using UnmapSubwindowsResponse = Response<void>;

  Future<void> UnmapSubwindows(
      const UnmapSubwindowsRequest& request);

  struct ConfigureWindowRequest {
    Window window{};
    ConfigWindow value_mask{};
    struct {
      int32_t x{};
      int32_t y{};
      uint32_t width{};
      uint32_t height{};
      uint32_t border_width{};
      Window sibling{};
      StackMode stack_mode{};
    } value_list;
  };

  using ConfigureWindowResponse = Response<void>;

  Future<void> ConfigureWindow(
      const ConfigureWindowRequest& request);

  struct CirculateWindowRequest {
    Circulate direction{};
    Window window{};
  };

  using CirculateWindowResponse = Response<void>;

  Future<void> CirculateWindow(
      const CirculateWindowRequest& request);

  struct GetGeometryRequest {
    Drawable drawable{};
  };

  struct GetGeometryReply {
    uint8_t depth{};
    Window root{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint16_t border_width{};
  };

  using GetGeometryResponse = Response<GetGeometryReply>;

  Future<GetGeometryReply> GetGeometry(
      const GetGeometryRequest& request);

  struct QueryTreeRequest {
    Window window{};
  };

  struct QueryTreeReply {
    Window root{};
    Window parent{};
    std::vector<Window> children{};
  };

  using QueryTreeResponse = Response<QueryTreeReply>;

  Future<QueryTreeReply> QueryTree(
      const QueryTreeRequest& request);

  struct InternAtomRequest {
    uint8_t only_if_exists{};
    std::string name{};
  };

  struct InternAtomReply {
    Atom atom{};
  };

  using InternAtomResponse = Response<InternAtomReply>;

  Future<InternAtomReply> InternAtom(
      const InternAtomRequest& request);

  struct GetAtomNameRequest {
    Atom atom{};
  };

  struct GetAtomNameReply {
    std::string name{};
  };

  using GetAtomNameResponse = Response<GetAtomNameReply>;

  Future<GetAtomNameReply> GetAtomName(
      const GetAtomNameRequest& request);

  struct ChangePropertyRequest {
    PropMode mode{};
    Window window{};
    Atom property{};
    Atom type{};
    uint8_t format{};
    uint32_t data_len{};
    std::vector<uint8_t> data{};
  };

  using ChangePropertyResponse = Response<void>;

  Future<void> ChangeProperty(
      const ChangePropertyRequest& request);

  struct DeletePropertyRequest {
    Window window{};
    Atom property{};
  };

  using DeletePropertyResponse = Response<void>;

  Future<void> DeleteProperty(
      const DeletePropertyRequest& request);

  struct GetPropertyRequest {
    uint8_t c_delete{};
    Window window{};
    Atom property{};
    Atom type{};
    uint32_t long_offset{};
    uint32_t long_length{};
  };

  struct GetPropertyReply {
    uint8_t format{};
    Atom type{};
    uint32_t bytes_after{};
    uint32_t value_len{};
    std::vector<uint8_t> value{};
  };

  using GetPropertyResponse = Response<GetPropertyReply>;

  Future<GetPropertyReply> GetProperty(
      const GetPropertyRequest& request);

  struct ListPropertiesRequest {
    Window window{};
  };

  struct ListPropertiesReply {
    uint16_t atoms_len{};
    std::vector<Atom> atoms{};
  };

  using ListPropertiesResponse = Response<ListPropertiesReply>;

  Future<ListPropertiesReply> ListProperties(
      const ListPropertiesRequest& request);

  struct SetSelectionOwnerRequest {
    Window owner{};
    Atom selection{};
    TimeStamp time{};
  };

  using SetSelectionOwnerResponse = Response<void>;

  Future<void> SetSelectionOwner(
      const SetSelectionOwnerRequest& request);

  struct GetSelectionOwnerRequest {
    Atom selection{};
  };

  struct GetSelectionOwnerReply {
    Window owner{};
  };

  using GetSelectionOwnerResponse = Response<GetSelectionOwnerReply>;

  Future<GetSelectionOwnerReply> GetSelectionOwner(
      const GetSelectionOwnerRequest& request);

  struct ConvertSelectionRequest {
    Window requestor{};
    Atom selection{};
    Atom target{};
    Atom property{};
    TimeStamp time{};
  };

  using ConvertSelectionResponse = Response<void>;

  Future<void> ConvertSelection(
      const ConvertSelectionRequest& request);

  struct SendEventRequest {
    uint8_t propagate{};
    Window destination{};
    EventMask event_mask{};
    std::array<char, 32> event{};
  };

  using SendEventResponse = Response<void>;

  Future<void> SendEvent(
      const SendEventRequest& request);

  struct GrabPointerRequest {
    uint8_t owner_events{};
    Window grab_window{};
    EventMask event_mask{};
    GrabMode pointer_mode{};
    GrabMode keyboard_mode{};
    Window confine_to{};
    Cursor cursor{};
    TimeStamp time{};
  };

  struct GrabPointerReply {
    GrabStatus status{};
  };

  using GrabPointerResponse = Response<GrabPointerReply>;

  Future<GrabPointerReply> GrabPointer(
      const GrabPointerRequest& request);

  struct UngrabPointerRequest {
    TimeStamp time{};
  };

  using UngrabPointerResponse = Response<void>;

  Future<void> UngrabPointer(
      const UngrabPointerRequest& request);

  struct GrabButtonRequest {
    uint8_t owner_events{};
    Window grab_window{};
    EventMask event_mask{};
    GrabMode pointer_mode{};
    GrabMode keyboard_mode{};
    Window confine_to{};
    Cursor cursor{};
    ButtonIndex button{};
    ModMask modifiers{};
  };

  using GrabButtonResponse = Response<void>;

  Future<void> GrabButton(
      const GrabButtonRequest& request);

  struct UngrabButtonRequest {
    ButtonIndex button{};
    Window grab_window{};
    ModMask modifiers{};
  };

  using UngrabButtonResponse = Response<void>;

  Future<void> UngrabButton(
      const UngrabButtonRequest& request);

  struct ChangeActivePointerGrabRequest {
    Cursor cursor{};
    TimeStamp time{};
    EventMask event_mask{};
  };

  using ChangeActivePointerGrabResponse = Response<void>;

  Future<void> ChangeActivePointerGrab(
      const ChangeActivePointerGrabRequest& request);

  struct GrabKeyboardRequest {
    uint8_t owner_events{};
    Window grab_window{};
    TimeStamp time{};
    GrabMode pointer_mode{};
    GrabMode keyboard_mode{};
  };

  struct GrabKeyboardReply {
    GrabStatus status{};
  };

  using GrabKeyboardResponse = Response<GrabKeyboardReply>;

  Future<GrabKeyboardReply> GrabKeyboard(
      const GrabKeyboardRequest& request);

  struct UngrabKeyboardRequest {
    TimeStamp time{};
  };

  using UngrabKeyboardResponse = Response<void>;

  Future<void> UngrabKeyboard(
      const UngrabKeyboardRequest& request);

  struct GrabKeyRequest {
    uint8_t owner_events{};
    Window grab_window{};
    ModMask modifiers{};
    KeyCode key{};
    GrabMode pointer_mode{};
    GrabMode keyboard_mode{};
  };

  using GrabKeyResponse = Response<void>;

  Future<void> GrabKey(
      const GrabKeyRequest& request);

  struct UngrabKeyRequest {
    KeyCode key{};
    Window grab_window{};
    ModMask modifiers{};
  };

  using UngrabKeyResponse = Response<void>;

  Future<void> UngrabKey(
      const UngrabKeyRequest& request);

  struct AllowEventsRequest {
    Allow mode{};
    TimeStamp time{};
  };

  using AllowEventsResponse = Response<void>;

  Future<void> AllowEvents(
      const AllowEventsRequest& request);

  struct GrabServerRequest {
  };

  using GrabServerResponse = Response<void>;

  Future<void> GrabServer(
      const GrabServerRequest& request);

  struct UngrabServerRequest {
  };

  using UngrabServerResponse = Response<void>;

  Future<void> UngrabServer(
      const UngrabServerRequest& request);

  struct QueryPointerRequest {
    Window window{};
  };

  struct QueryPointerReply {
    uint8_t same_screen{};
    Window root{};
    Window child{};
    int16_t root_x{};
    int16_t root_y{};
    int16_t win_x{};
    int16_t win_y{};
    KeyButMask mask{};
  };

  using QueryPointerResponse = Response<QueryPointerReply>;

  Future<QueryPointerReply> QueryPointer(
      const QueryPointerRequest& request);

  struct GetMotionEventsRequest {
    Window window{};
    TimeStamp start{};
    TimeStamp stop{};
  };

  struct GetMotionEventsReply {
    std::vector<TimeCoord> events{};
  };

  using GetMotionEventsResponse = Response<GetMotionEventsReply>;

  Future<GetMotionEventsReply> GetMotionEvents(
      const GetMotionEventsRequest& request);

  struct TranslateCoordinatesRequest {
    Window src_window{};
    Window dst_window{};
    int16_t src_x{};
    int16_t src_y{};
  };

  struct TranslateCoordinatesReply {
    uint8_t same_screen{};
    Window child{};
    int16_t dst_x{};
    int16_t dst_y{};
  };

  using TranslateCoordinatesResponse = Response<TranslateCoordinatesReply>;

  Future<TranslateCoordinatesReply> TranslateCoordinates(
      const TranslateCoordinatesRequest& request);

  struct WarpPointerRequest {
    Window src_window{};
    Window dst_window{};
    int16_t src_x{};
    int16_t src_y{};
    uint16_t src_width{};
    uint16_t src_height{};
    int16_t dst_x{};
    int16_t dst_y{};
  };

  using WarpPointerResponse = Response<void>;

  Future<void> WarpPointer(
      const WarpPointerRequest& request);

  struct SetInputFocusRequest {
    InputFocus revert_to{};
    Window focus{};
    TimeStamp time{};
  };

  using SetInputFocusResponse = Response<void>;

  Future<void> SetInputFocus(
      const SetInputFocusRequest& request);

  struct GetInputFocusRequest {
  };

  struct GetInputFocusReply {
    InputFocus revert_to{};
    Window focus{};
  };

  using GetInputFocusResponse = Response<GetInputFocusReply>;

  Future<GetInputFocusReply> GetInputFocus(
      const GetInputFocusRequest& request);

  struct QueryKeymapRequest {
  };

  struct QueryKeymapReply {
    std::array<uint8_t, 32> keys{};
  };

  using QueryKeymapResponse = Response<QueryKeymapReply>;

  Future<QueryKeymapReply> QueryKeymap(
      const QueryKeymapRequest& request);

  struct OpenFontRequest {
    Font fid{};
    std::string name{};
  };

  using OpenFontResponse = Response<void>;

  Future<void> OpenFont(
      const OpenFontRequest& request);

  struct CloseFontRequest {
    Font font{};
  };

  using CloseFontResponse = Response<void>;

  Future<void> CloseFont(
      const CloseFontRequest& request);

  struct QueryFontRequest {
    Fontable font{};
  };

  struct QueryFontReply {
    CharInfo min_bounds{};
    CharInfo max_bounds{};
    uint16_t min_char_or_byte2{};
    uint16_t max_char_or_byte2{};
    uint16_t default_char{};
    FontDraw draw_direction{};
    uint8_t min_byte1{};
    uint8_t max_byte1{};
    uint8_t all_chars_exist{};
    int16_t font_ascent{};
    int16_t font_descent{};
    std::vector<FontProperty> properties{};
    std::vector<CharInfo> char_infos{};
  };

  using QueryFontResponse = Response<QueryFontReply>;

  Future<QueryFontReply> QueryFont(
      const QueryFontRequest& request);

  struct QueryTextExtentsRequest {
    Fontable font{};
    std::vector<Char16> string{};
  };

  struct QueryTextExtentsReply {
    FontDraw draw_direction{};
    int16_t font_ascent{};
    int16_t font_descent{};
    int16_t overall_ascent{};
    int16_t overall_descent{};
    int32_t overall_width{};
    int32_t overall_left{};
    int32_t overall_right{};
  };

  using QueryTextExtentsResponse = Response<QueryTextExtentsReply>;

  Future<QueryTextExtentsReply> QueryTextExtents(
      const QueryTextExtentsRequest& request);

  struct ListFontsRequest {
    uint16_t max_names{};
    std::string pattern{};
  };

  struct ListFontsReply {
    uint16_t names_len{};
    std::vector<Str> names{};
  };

  using ListFontsResponse = Response<ListFontsReply>;

  Future<ListFontsReply> ListFonts(
      const ListFontsRequest& request);

  struct ListFontsWithInfoRequest {
    uint16_t max_names{};
    std::string pattern{};
  };

  struct ListFontsWithInfoReply {
    CharInfo min_bounds{};
    CharInfo max_bounds{};
    uint16_t min_char_or_byte2{};
    uint16_t max_char_or_byte2{};
    uint16_t default_char{};
    FontDraw draw_direction{};
    uint8_t min_byte1{};
    uint8_t max_byte1{};
    uint8_t all_chars_exist{};
    int16_t font_ascent{};
    int16_t font_descent{};
    uint32_t replies_hint{};
    std::vector<FontProperty> properties{};
    std::string name{};
  };

  using ListFontsWithInfoResponse = Response<ListFontsWithInfoReply>;

  Future<ListFontsWithInfoReply> ListFontsWithInfo(
      const ListFontsWithInfoRequest& request);

  struct SetFontPathRequest {
    std::vector<Str> font{};
  };

  using SetFontPathResponse = Response<void>;

  Future<void> SetFontPath(
      const SetFontPathRequest& request);

  struct GetFontPathRequest {
  };

  struct GetFontPathReply {
    std::vector<Str> path{};
  };

  using GetFontPathResponse = Response<GetFontPathReply>;

  Future<GetFontPathReply> GetFontPath(
      const GetFontPathRequest& request);

  struct CreatePixmapRequest {
    uint8_t depth{};
    Pixmap pid{};
    Drawable drawable{};
    uint16_t width{};
    uint16_t height{};
  };

  using CreatePixmapResponse = Response<void>;

  Future<void> CreatePixmap(
      const CreatePixmapRequest& request);

  struct FreePixmapRequest {
    Pixmap pixmap{};
  };

  using FreePixmapResponse = Response<void>;

  Future<void> FreePixmap(
      const FreePixmapRequest& request);

  struct CreateGCRequest {
    GraphicsContext cid{};
    Drawable drawable{};
    GraphicsContextAttribute value_mask{};
    struct {
      Gx function{};
      uint32_t plane_mask{};
      uint32_t foreground{};
      uint32_t background{};
      uint32_t line_width{};
      LineStyle line_style{};
      CapStyle cap_style{};
      JoinStyle join_style{};
      FillStyle fill_style{};
      FillRule fill_rule{};
      Pixmap tile{};
      Pixmap stipple{};
      int32_t tile_stipple_x_origin{};
      int32_t tile_stipple_y_origin{};
      Font font{};
      SubwindowMode subwindow_mode{};
      Bool32 graphics_exposures{};
      int32_t clip_x_origin{};
      int32_t clip_y_origin{};
      Pixmap clip_mask{};
      uint32_t dash_offset{};
      uint32_t dashes{};
      ArcMode arc_mode{};
    } value_list;
  };

  using CreateGCResponse = Response<void>;

  Future<void> CreateGC(
      const CreateGCRequest& request);

  struct ChangeGCRequest {
    GraphicsContext gc{};
    GraphicsContextAttribute value_mask{};
    struct {
      Gx function{};
      uint32_t plane_mask{};
      uint32_t foreground{};
      uint32_t background{};
      uint32_t line_width{};
      LineStyle line_style{};
      CapStyle cap_style{};
      JoinStyle join_style{};
      FillStyle fill_style{};
      FillRule fill_rule{};
      Pixmap tile{};
      Pixmap stipple{};
      int32_t tile_stipple_x_origin{};
      int32_t tile_stipple_y_origin{};
      Font font{};
      SubwindowMode subwindow_mode{};
      Bool32 graphics_exposures{};
      int32_t clip_x_origin{};
      int32_t clip_y_origin{};
      Pixmap clip_mask{};
      uint32_t dash_offset{};
      uint32_t dashes{};
      ArcMode arc_mode{};
    } value_list;
  };

  using ChangeGCResponse = Response<void>;

  Future<void> ChangeGC(
      const ChangeGCRequest& request);

  struct CopyGCRequest {
    GraphicsContext src_gc{};
    GraphicsContext dst_gc{};
    GraphicsContextAttribute value_mask{};
  };

  using CopyGCResponse = Response<void>;

  Future<void> CopyGC(
      const CopyGCRequest& request);

  struct SetDashesRequest {
    GraphicsContext gc{};
    uint16_t dash_offset{};
    uint16_t dashes_len{};
    std::vector<uint8_t> dashes{};
  };

  using SetDashesResponse = Response<void>;

  Future<void> SetDashes(
      const SetDashesRequest& request);

  struct SetClipRectanglesRequest {
    ClipOrdering ordering{};
    GraphicsContext gc{};
    int16_t clip_x_origin{};
    int16_t clip_y_origin{};
    std::vector<Rectangle> rectangles{};
  };

  using SetClipRectanglesResponse = Response<void>;

  Future<void> SetClipRectangles(
      const SetClipRectanglesRequest& request);

  struct FreeGCRequest {
    GraphicsContext gc{};
  };

  using FreeGCResponse = Response<void>;

  Future<void> FreeGC(
      const FreeGCRequest& request);

  struct ClearAreaRequest {
    uint8_t exposures{};
    Window window{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
  };

  using ClearAreaResponse = Response<void>;

  Future<void> ClearArea(
      const ClearAreaRequest& request);

  struct CopyAreaRequest {
    Drawable src_drawable{};
    Drawable dst_drawable{};
    GraphicsContext gc{};
    int16_t src_x{};
    int16_t src_y{};
    int16_t dst_x{};
    int16_t dst_y{};
    uint16_t width{};
    uint16_t height{};
  };

  using CopyAreaResponse = Response<void>;

  Future<void> CopyArea(
      const CopyAreaRequest& request);

  struct CopyPlaneRequest {
    Drawable src_drawable{};
    Drawable dst_drawable{};
    GraphicsContext gc{};
    int16_t src_x{};
    int16_t src_y{};
    int16_t dst_x{};
    int16_t dst_y{};
    uint16_t width{};
    uint16_t height{};
    uint32_t bit_plane{};
  };

  using CopyPlaneResponse = Response<void>;

  Future<void> CopyPlane(
      const CopyPlaneRequest& request);

  struct PolyPointRequest {
    CoordMode coordinate_mode{};
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<Point> points{};
  };

  using PolyPointResponse = Response<void>;

  Future<void> PolyPoint(
      const PolyPointRequest& request);

  struct PolyLineRequest {
    CoordMode coordinate_mode{};
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<Point> points{};
  };

  using PolyLineResponse = Response<void>;

  Future<void> PolyLine(
      const PolyLineRequest& request);

  struct PolySegmentRequest {
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<SEGMENT> segments{};
  };

  using PolySegmentResponse = Response<void>;

  Future<void> PolySegment(
      const PolySegmentRequest& request);

  struct PolyRectangleRequest {
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<Rectangle> rectangles{};
  };

  using PolyRectangleResponse = Response<void>;

  Future<void> PolyRectangle(
      const PolyRectangleRequest& request);

  struct PolyArcRequest {
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<Arc> arcs{};
  };

  using PolyArcResponse = Response<void>;

  Future<void> PolyArc(
      const PolyArcRequest& request);

  struct FillPolyRequest {
    Drawable drawable{};
    GraphicsContext gc{};
    PolyShape shape{};
    CoordMode coordinate_mode{};
    std::vector<Point> points{};
  };

  using FillPolyResponse = Response<void>;

  Future<void> FillPoly(
      const FillPolyRequest& request);

  struct PolyFillRectangleRequest {
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<Rectangle> rectangles{};
  };

  using PolyFillRectangleResponse = Response<void>;

  Future<void> PolyFillRectangle(
      const PolyFillRectangleRequest& request);

  struct PolyFillArcRequest {
    Drawable drawable{};
    GraphicsContext gc{};
    std::vector<Arc> arcs{};
  };

  using PolyFillArcResponse = Response<void>;

  Future<void> PolyFillArc(
      const PolyFillArcRequest& request);

  struct PutImageRequest {
    ImageFormat format{};
    Drawable drawable{};
    GraphicsContext gc{};
    uint16_t width{};
    uint16_t height{};
    int16_t dst_x{};
    int16_t dst_y{};
    uint8_t left_pad{};
    uint8_t depth{};
    std::vector<uint8_t> data{};
  };

  using PutImageResponse = Response<void>;

  Future<void> PutImage(
      const PutImageRequest& request);

  struct GetImageRequest {
    ImageFormat format{};
    Drawable drawable{};
    int16_t x{};
    int16_t y{};
    uint16_t width{};
    uint16_t height{};
    uint32_t plane_mask{};
  };

  struct GetImageReply {
    uint8_t depth{};
    VisualId visual{};
    std::vector<uint8_t> data{};
  };

  using GetImageResponse = Response<GetImageReply>;

  Future<GetImageReply> GetImage(
      const GetImageRequest& request);

  struct PolyText8Request {
    Drawable drawable{};
    GraphicsContext gc{};
    int16_t x{};
    int16_t y{};
    std::vector<uint8_t> items{};
  };

  using PolyText8Response = Response<void>;

  Future<void> PolyText8(
      const PolyText8Request& request);

  struct PolyText16Request {
    Drawable drawable{};
    GraphicsContext gc{};
    int16_t x{};
    int16_t y{};
    std::vector<uint8_t> items{};
  };

  using PolyText16Response = Response<void>;

  Future<void> PolyText16(
      const PolyText16Request& request);

  struct ImageText8Request {
    uint8_t string_len{};
    Drawable drawable{};
    GraphicsContext gc{};
    int16_t x{};
    int16_t y{};
    std::string string{};
  };

  using ImageText8Response = Response<void>;

  Future<void> ImageText8(
      const ImageText8Request& request);

  struct ImageText16Request {
    uint8_t string_len{};
    Drawable drawable{};
    GraphicsContext gc{};
    int16_t x{};
    int16_t y{};
    std::vector<Char16> string{};
  };

  using ImageText16Response = Response<void>;

  Future<void> ImageText16(
      const ImageText16Request& request);

  struct CreateColormapRequest {
    ColormapAlloc alloc{};
    ColorMap mid{};
    Window window{};
    VisualId visual{};
  };

  using CreateColormapResponse = Response<void>;

  Future<void> CreateColormap(
      const CreateColormapRequest& request);

  struct FreeColormapRequest {
    ColorMap cmap{};
  };

  using FreeColormapResponse = Response<void>;

  Future<void> FreeColormap(
      const FreeColormapRequest& request);

  struct CopyColormapAndFreeRequest {
    ColorMap mid{};
    ColorMap src_cmap{};
  };

  using CopyColormapAndFreeResponse = Response<void>;

  Future<void> CopyColormapAndFree(
      const CopyColormapAndFreeRequest& request);

  struct InstallColormapRequest {
    ColorMap cmap{};
  };

  using InstallColormapResponse = Response<void>;

  Future<void> InstallColormap(
      const InstallColormapRequest& request);

  struct UninstallColormapRequest {
    ColorMap cmap{};
  };

  using UninstallColormapResponse = Response<void>;

  Future<void> UninstallColormap(
      const UninstallColormapRequest& request);

  struct ListInstalledColormapsRequest {
    Window window{};
  };

  struct ListInstalledColormapsReply {
    uint16_t cmaps_len{};
    std::vector<ColorMap> cmaps{};
  };

  using ListInstalledColormapsResponse = Response<ListInstalledColormapsReply>;

  Future<ListInstalledColormapsReply> ListInstalledColormaps(
      const ListInstalledColormapsRequest& request);

  struct AllocColorRequest {
    ColorMap cmap{};
    uint16_t red{};
    uint16_t green{};
    uint16_t blue{};
  };

  struct AllocColorReply {
    uint16_t red{};
    uint16_t green{};
    uint16_t blue{};
    uint32_t pixel{};
  };

  using AllocColorResponse = Response<AllocColorReply>;

  Future<AllocColorReply> AllocColor(
      const AllocColorRequest& request);

  struct AllocNamedColorRequest {
    ColorMap cmap{};
    std::string name{};
  };

  struct AllocNamedColorReply {
    uint32_t pixel{};
    uint16_t exact_red{};
    uint16_t exact_green{};
    uint16_t exact_blue{};
    uint16_t visual_red{};
    uint16_t visual_green{};
    uint16_t visual_blue{};
  };

  using AllocNamedColorResponse = Response<AllocNamedColorReply>;

  Future<AllocNamedColorReply> AllocNamedColor(
      const AllocNamedColorRequest& request);

  struct AllocColorCellsRequest {
    uint8_t contiguous{};
    ColorMap cmap{};
    uint16_t colors{};
    uint16_t planes{};
  };

  struct AllocColorCellsReply {
    std::vector<uint32_t> pixels{};
    std::vector<uint32_t> masks{};
  };

  using AllocColorCellsResponse = Response<AllocColorCellsReply>;

  Future<AllocColorCellsReply> AllocColorCells(
      const AllocColorCellsRequest& request);

  struct AllocColorPlanesRequest {
    uint8_t contiguous{};
    ColorMap cmap{};
    uint16_t colors{};
    uint16_t reds{};
    uint16_t greens{};
    uint16_t blues{};
  };

  struct AllocColorPlanesReply {
    uint32_t red_mask{};
    uint32_t green_mask{};
    uint32_t blue_mask{};
    std::vector<uint32_t> pixels{};
  };

  using AllocColorPlanesResponse = Response<AllocColorPlanesReply>;

  Future<AllocColorPlanesReply> AllocColorPlanes(
      const AllocColorPlanesRequest& request);

  struct FreeColorsRequest {
    ColorMap cmap{};
    uint32_t plane_mask{};
    std::vector<uint32_t> pixels{};
  };

  using FreeColorsResponse = Response<void>;

  Future<void> FreeColors(
      const FreeColorsRequest& request);

  struct StoreColorsRequest {
    ColorMap cmap{};
    std::vector<ColorItem> items{};
  };

  using StoreColorsResponse = Response<void>;

  Future<void> StoreColors(
      const StoreColorsRequest& request);

  struct StoreNamedColorRequest {
    ColorFlag flags{};
    ColorMap cmap{};
    uint32_t pixel{};
    std::string name{};
  };

  using StoreNamedColorResponse = Response<void>;

  Future<void> StoreNamedColor(
      const StoreNamedColorRequest& request);

  struct QueryColorsRequest {
    ColorMap cmap{};
    std::vector<uint32_t> pixels{};
  };

  struct QueryColorsReply {
    uint16_t colors_len{};
    std::vector<Rgb> colors{};
  };

  using QueryColorsResponse = Response<QueryColorsReply>;

  Future<QueryColorsReply> QueryColors(
      const QueryColorsRequest& request);

  struct LookupColorRequest {
    ColorMap cmap{};
    std::string name{};
  };

  struct LookupColorReply {
    uint16_t exact_red{};
    uint16_t exact_green{};
    uint16_t exact_blue{};
    uint16_t visual_red{};
    uint16_t visual_green{};
    uint16_t visual_blue{};
  };

  using LookupColorResponse = Response<LookupColorReply>;

  Future<LookupColorReply> LookupColor(
      const LookupColorRequest& request);

  struct CreateCursorRequest {
    Cursor cid{};
    Pixmap source{};
    Pixmap mask{};
    uint16_t fore_red{};
    uint16_t fore_green{};
    uint16_t fore_blue{};
    uint16_t back_red{};
    uint16_t back_green{};
    uint16_t back_blue{};
    uint16_t x{};
    uint16_t y{};
  };

  using CreateCursorResponse = Response<void>;

  Future<void> CreateCursor(
      const CreateCursorRequest& request);

  struct CreateGlyphCursorRequest {
    Cursor cid{};
    Font source_font{};
    Font mask_font{};
    uint16_t source_char{};
    uint16_t mask_char{};
    uint16_t fore_red{};
    uint16_t fore_green{};
    uint16_t fore_blue{};
    uint16_t back_red{};
    uint16_t back_green{};
    uint16_t back_blue{};
  };

  using CreateGlyphCursorResponse = Response<void>;

  Future<void> CreateGlyphCursor(
      const CreateGlyphCursorRequest& request);

  struct FreeCursorRequest {
    Cursor cursor{};
  };

  using FreeCursorResponse = Response<void>;

  Future<void> FreeCursor(
      const FreeCursorRequest& request);

  struct RecolorCursorRequest {
    Cursor cursor{};
    uint16_t fore_red{};
    uint16_t fore_green{};
    uint16_t fore_blue{};
    uint16_t back_red{};
    uint16_t back_green{};
    uint16_t back_blue{};
  };

  using RecolorCursorResponse = Response<void>;

  Future<void> RecolorCursor(
      const RecolorCursorRequest& request);

  struct QueryBestSizeRequest {
    QueryShapeOf c_class{};
    Drawable drawable{};
    uint16_t width{};
    uint16_t height{};
  };

  struct QueryBestSizeReply {
    uint16_t width{};
    uint16_t height{};
  };

  using QueryBestSizeResponse = Response<QueryBestSizeReply>;

  Future<QueryBestSizeReply> QueryBestSize(
      const QueryBestSizeRequest& request);

  struct QueryExtensionRequest {
    std::string name{};
  };

  struct QueryExtensionReply {
    uint8_t present{};
    uint8_t major_opcode{};
    uint8_t first_event{};
    uint8_t first_error{};
  };

  using QueryExtensionResponse = Response<QueryExtensionReply>;

  Future<QueryExtensionReply> QueryExtension(
      const QueryExtensionRequest& request);

  struct ListExtensionsRequest {
  };

  struct ListExtensionsReply {
    std::vector<Str> names{};
  };

  using ListExtensionsResponse = Response<ListExtensionsReply>;

  Future<ListExtensionsReply> ListExtensions(
      const ListExtensionsRequest& request);

  struct ChangeKeyboardMappingRequest {
    uint8_t keycode_count{};
    KeyCode first_keycode{};
    uint8_t keysyms_per_keycode{};
    std::vector<KeySym> keysyms{};
  };

  using ChangeKeyboardMappingResponse = Response<void>;

  Future<void> ChangeKeyboardMapping(
      const ChangeKeyboardMappingRequest& request);

  struct GetKeyboardMappingRequest {
    KeyCode first_keycode{};
    uint8_t count{};
  };

  struct GetKeyboardMappingReply {
    uint8_t keysyms_per_keycode{};
    std::vector<KeySym> keysyms{};
  };

  using GetKeyboardMappingResponse = Response<GetKeyboardMappingReply>;

  Future<GetKeyboardMappingReply> GetKeyboardMapping(
      const GetKeyboardMappingRequest& request);

  struct ChangeKeyboardControlRequest {
    Keyboard value_mask{};
    struct {
      int32_t key_click_percent{};
      int32_t bell_percent{};
      int32_t bell_pitch{};
      int32_t bell_duration{};
      uint32_t led{};
      LedMode led_mode{};
      KeyCode32 key{};
      AutoRepeatMode auto_repeat_mode{};
    } value_list;
  };

  using ChangeKeyboardControlResponse = Response<void>;

  Future<void> ChangeKeyboardControl(
      const ChangeKeyboardControlRequest& request);

  struct GetKeyboardControlRequest {
  };

  struct GetKeyboardControlReply {
    AutoRepeatMode global_auto_repeat{};
    uint32_t led_mask{};
    uint8_t key_click_percent{};
    uint8_t bell_percent{};
    uint16_t bell_pitch{};
    uint16_t bell_duration{};
    std::array<uint8_t, 32> auto_repeats{};
  };

  using GetKeyboardControlResponse = Response<GetKeyboardControlReply>;

  Future<GetKeyboardControlReply> GetKeyboardControl(
      const GetKeyboardControlRequest& request);

  struct BellRequest {
    int8_t percent{};
  };

  using BellResponse = Response<void>;

  Future<void> Bell(
      const BellRequest& request);

  struct ChangePointerControlRequest {
    int16_t acceleration_numerator{};
    int16_t acceleration_denominator{};
    int16_t threshold{};
    uint8_t do_acceleration{};
    uint8_t do_threshold{};
  };

  using ChangePointerControlResponse = Response<void>;

  Future<void> ChangePointerControl(
      const ChangePointerControlRequest& request);

  struct GetPointerControlRequest {
  };

  struct GetPointerControlReply {
    uint16_t acceleration_numerator{};
    uint16_t acceleration_denominator{};
    uint16_t threshold{};
  };

  using GetPointerControlResponse = Response<GetPointerControlReply>;

  Future<GetPointerControlReply> GetPointerControl(
      const GetPointerControlRequest& request);

  struct SetScreenSaverRequest {
    int16_t timeout{};
    int16_t interval{};
    Blanking prefer_blanking{};
    Exposures allow_exposures{};
  };

  using SetScreenSaverResponse = Response<void>;

  Future<void> SetScreenSaver(
      const SetScreenSaverRequest& request);

  struct GetScreenSaverRequest {
  };

  struct GetScreenSaverReply {
    uint16_t timeout{};
    uint16_t interval{};
    Blanking prefer_blanking{};
    Exposures allow_exposures{};
  };

  using GetScreenSaverResponse = Response<GetScreenSaverReply>;

  Future<GetScreenSaverReply> GetScreenSaver(
      const GetScreenSaverRequest& request);

  struct ChangeHostsRequest {
    HostMode mode{};
    Family family{};
    uint16_t address_len{};
    std::vector<uint8_t> address{};
  };

  using ChangeHostsResponse = Response<void>;

  Future<void> ChangeHosts(
      const ChangeHostsRequest& request);

  struct ListHostsRequest {
  };

  struct ListHostsReply {
    AccessControl mode{};
    std::vector<Host> hosts{};
  };

  using ListHostsResponse = Response<ListHostsReply>;

  Future<ListHostsReply> ListHosts(
      const ListHostsRequest& request);

  struct SetAccessControlRequest {
    AccessControl mode{};
  };

  using SetAccessControlResponse = Response<void>;

  Future<void> SetAccessControl(
      const SetAccessControlRequest& request);

  struct SetCloseDownModeRequest {
    CloseDown mode{};
  };

  using SetCloseDownModeResponse = Response<void>;

  Future<void> SetCloseDownMode(
      const SetCloseDownModeRequest& request);

  struct KillClientRequest {
    uint32_t resource{};
  };

  using KillClientResponse = Response<void>;

  Future<void> KillClient(
      const KillClientRequest& request);

  struct RotatePropertiesRequest {
    Window window{};
    uint16_t atoms_len{};
    int16_t delta{};
    std::vector<Atom> atoms{};
  };

  using RotatePropertiesResponse = Response<void>;

  Future<void> RotateProperties(
      const RotatePropertiesRequest& request);

  struct ForceScreenSaverRequest {
    ScreenSaver mode{};
  };

  using ForceScreenSaverResponse = Response<void>;

  Future<void> ForceScreenSaver(
      const ForceScreenSaverRequest& request);

  struct SetPointerMappingRequest {
    std::vector<uint8_t> map{};
  };

  struct SetPointerMappingReply {
    MappingStatus status{};
  };

  using SetPointerMappingResponse = Response<SetPointerMappingReply>;

  Future<SetPointerMappingReply> SetPointerMapping(
      const SetPointerMappingRequest& request);

  struct GetPointerMappingRequest {
  };

  struct GetPointerMappingReply {
    uint8_t map_len{};
    std::vector<uint8_t> map{};
  };

  using GetPointerMappingResponse = Response<GetPointerMappingReply>;

  Future<GetPointerMappingReply> GetPointerMapping(
      const GetPointerMappingRequest& request);

  struct SetModifierMappingRequest {
    uint8_t keycodes_per_modifier{};
    std::vector<KeyCode> keycodes{};
  };

  struct SetModifierMappingReply {
    MappingStatus status{};
  };

  using SetModifierMappingResponse = Response<SetModifierMappingReply>;

  Future<SetModifierMappingReply> SetModifierMapping(
      const SetModifierMappingRequest& request);

  struct GetModifierMappingRequest {
  };

  struct GetModifierMappingReply {
    uint8_t keycodes_per_modifier{};
    std::vector<KeyCode> keycodes{};
  };

  using GetModifierMappingResponse = Response<GetModifierMappingReply>;

  Future<GetModifierMappingReply> GetModifierMapping(
      const GetModifierMappingRequest& request);

  struct NoOperationRequest {
  };

  using NoOperationResponse = Response<void>;

  Future<void> NoOperation(
      const NoOperationRequest& request);

  private:
  XDisplay* const display_;
};

}  // namespace x11

#endif  // GEN_UI_GFX_X_XPROTO_H_
